from datetime import datetime

from dateutil import parser


class DateParser:
    @staticmethod
    def datetime_from_str(value: str):
        if isinstance(value, str):
            value = parser.parse(value)
        return value

    @staticmethod
    def str_from_datetime(value: datetime):
        return str(value) if value is not None else None



import hmac
import hashlib


class BodyDigestSignature(object):
    def __init__(self, secret, organization, header="Sign", algorithm=hashlib.sha1):
        self.secret = secret
        self.organization = organization
        self.header = header
        self.algorithm = algorithm

    def __call__(self, request):
        msg = self._format_message(request).encode("latin1")
        signature = hmac.new(key=self.secret, msg=msg, digestmod=self.algorithm)
        request.headers[self.header] = signature.hexdigest()
        return request

    def _format_message(self, request):
        event_name = request.headers.get(f"X-{self.organization}-Event")
        event_version = request.headers.get(f"X-{self.organization}-Event-Version")
        timestamp = request.headers.get(f"X-{self.organization}-Request-Timestamp")
        body = request.body

        return f"{event_name}.{event_version}:{timestamp}:{body}"



def rabbitmq_is_running_locally() -> bool:
    try:
        import pika  # noqa: F401

        connection = pika.BlockingConnection(
            pika.ConnectionParameters(host="localhost")
        )
        is_running_locally = connection.is_open
        connection.close()
        return is_running_locally
    except:  # noqa: E722
        return False



from meiga import Error

from petisco.domain.base_object import BaseObject
from dataclasses_json import config
from dataclasses import field

ValueObject = BaseObject

ValueObjectError = Error


def value_object_field(value_object_class=ValueObject):
    def _field():
        return field(
            metadata=config(
                encoder=lambda result: result.value,
                decoder=lambda value: value_object_class(value),
            )
        )

    return _field()



from typing import Dict

from abc import ABCMeta, abstractmethod


class IEventChaos:

    __metaclass__ = ABCMeta

    @abstractmethod
    def info(self) -> Dict:
        raise NotImplementedError

    @abstractmethod
    def nack_simulation(self, *args, **kwargs) -> bool:
        raise NotImplementedError

    @abstractmethod
    def failure_simulation(self, *args, **kwargs) -> bool:
        raise NotImplementedError

    @abstractmethod
    def delay(self):
        raise NotImplementedError



from petisco.persistence.elastic.elastic_database import ElasticDatabase
from petisco.persistence.persistence import Persistence


def elastic_is_running_locally() -> bool:
    try:
        database = ElasticDatabase.local_connection_checker()
        Persistence().add(database)
        Persistence().create()
        is_running_locally = Persistence.is_available(database.name).ping()
    except:  # noqa: E722
        is_running_locally = False
    Persistence().remove("test", skip_if_not_exist=True)
    return is_running_locally



from petisco import RabbitMqConnector
from petisco.event.configurer.infrastructure.rabbitmq_declarer import RabbitMqDeclarer

from tests.modules.event.mothers.defaults import DEFAULT_ORGANIZATION, DEFAULT_SERVICE


class RabbitMqDeclarerMother:
    @staticmethod
    def default(connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqDeclarer(
            connector=connector,
            channel_name=f"{DEFAULT_ORGANIZATION}.{DEFAULT_SERVICE}",
        )



import pytest
import requests
import requests_mock
from meiga.assertions import assert_success, assert_failure

from petisco import (
    Request,
    MissingSchemaRequestError,
    TimeoutRequestError,
    ConnectionRequestError,
    BadRequestError,
    UnauthorizedRequestError,
    UnknownRequestError,
)


@pytest.fixture
def given_any_url():
    return "http://alicebiometrics.com"


@pytest.fixture
def given_invalid_url():
    return "invalid_url"


@pytest.mark.integration
def test_should_success_when_request_get(given_any_url):
    with requests_mock.Mocker() as m:
        m.get(given_any_url)
        result = Request.get(given_any_url)
        assert_success(result)


@pytest.mark.integration
def test_should_success_when_request_post(given_any_url):
    with requests_mock.Mocker() as m:
        m.post(given_any_url)
        result = Request.post(given_any_url)
        assert_success(result)


@pytest.mark.integration
def test_should_success_when_request_patch(given_any_url):
    with requests_mock.Mocker() as m:
        m.patch(given_any_url)
        result = Request.patch(given_any_url)
        assert_success(result)


@pytest.mark.integration
def test_should_success_when_request_delete(given_any_url):
    with requests_mock.Mocker() as m:
        m.delete(given_any_url)
        result = Request.delete(given_any_url)
        assert_success(result)


@pytest.mark.integration
@pytest.mark.parametrize("method", ["GET", "POST", "PATCH", "DELETE"])
def test_should_fail_when_request_method_raise_unauthorized_error(
    method, given_any_url
):
    with requests_mock.Mocker() as m:
        m.register_uri(method, given_any_url, status_code=401)
        result = Request.execute(given_any_url, method)
        assert_failure(result, value_is_instance_of=UnauthorizedRequestError)
        assert result.value.status_code == 401


@pytest.mark.integration
@pytest.mark.parametrize("method", ["GET", "POST", "PATCH", "DELETE"])
def test_should_fail_when_request_method_returns_missing_schema_error(
    method, given_any_url
):
    with requests_mock.Mocker() as m:
        m.register_uri(method, given_any_url, exc=requests.exceptions.MissingSchema)
        result = Request.execute(given_any_url, method)
        assert_failure(result, value_is_instance_of=MissingSchemaRequestError)
        assert result.value.status_code == 422


@pytest.mark.integration
@pytest.mark.parametrize("method", ["GET", "POST", "PATCH", "DELETE"])
def test_should_fail_when_request_method_returns_timeout_error(method, given_any_url):
    with requests_mock.Mocker() as m:
        m.register_uri(method, given_any_url, exc=requests.exceptions.ConnectTimeout)
        result = Request.execute(given_any_url, method)
        assert_failure(result, value_is_instance_of=TimeoutRequestError)
        assert result.value.status_code == 408


@pytest.mark.integration
@pytest.mark.parametrize("method", ["GET", "POST", "PATCH", "DELETE"])
def test_should_fail_when_request_method_returns_connection_error(
    method, given_any_url
):
    with requests_mock.Mocker() as m:
        m.register_uri(method, given_any_url, exc=requests.exceptions.ConnectionError)
        result = Request.execute(given_any_url, method)
        assert_failure(result, value_is_instance_of=ConnectionRequestError)
        assert result.value.status_code == 503


@pytest.mark.integration
@pytest.mark.parametrize("method", ["GET", "POST", "PATCH", "DELETE"])
def test_should_fail_when_request_method_returns_bad_request_error(
    method, given_any_url
):
    with requests_mock.Mocker() as m:
        m.register_uri(method, given_any_url, status_code=400)
        result = Request.execute(given_any_url, method)
        assert_failure(result, value_is_instance_of=BadRequestError)
        assert result.value.status_code == 400


@pytest.mark.integration
@pytest.mark.parametrize(
    "method,status_code",
    [
        ("GET", 100),
        ("POST", 100),
        ("PATCH", 100),
        ("DELETE", 100),
        ("GET", 300),
        ("POST", 300),
        ("PATCH", 300),
        ("DELETE", 300),
        ("GET", 500),
        ("POST", 500),
        ("PATCH", 500),
        ("DELETE", 500),
    ],
)
def test_should_fail_when_request_method_returns_unknown_error(
    method, status_code, given_any_url
):
    with requests_mock.Mocker() as m:
        m.register_uri(method, given_any_url, status_code=status_code)
        result = Request.execute(given_any_url, method)
        assert_failure(result, value_is_instance_of=UnknownRequestError)
        assert result.value.status_code == status_code


@pytest.mark.integration
@pytest.mark.parametrize("method", ["GET", "POST", "PATCH", "DELETE"])
def test_should_fail_when_request_method_returns_unknown_error_when_raise_an_uncontrolled_exception(
    method, given_any_url
):
    with requests_mock.Mocker() as m:
        m.register_uri(method, given_any_url, exc=InterruptedError)
        result = Request.execute(given_any_url, method)
        assert_failure(result, value_is_instance_of=UnknownRequestError)
        assert result.value.status_code == 500


@pytest.mark.integration
@pytest.mark.parametrize(
    "headers,content,json,expected_headers,expected_content",
    [
        ({}, b"", None, None, None),
        ({}, None, {}, None, {}),
        ({}, b'{"body": "petisco"}', None, None, {"body": "petisco"}),
        ({"header": "petisco"}, b"", None, {"header": "petisco"}, None),
        ({}, None, {"body": "petisco"}, None, {"body": "petisco"}),
        (
            {"header": "petisco"},
            b'{"body": "petisco"}',
            None,
            {"header": "petisco"},
            {"body": "petisco"},
        ),
        ({}, None, {"body": "petisco"}, None, {"body": "petisco"}),
    ],
)
def test_should_fail_with_expected_values_when_use_request(
    headers, content, json, expected_headers, expected_content, given_any_url
):
    with requests_mock.Mocker() as m:
        m.register_uri(
            "GET",
            given_any_url,
            headers=headers,
            content=content,
            json=json,
            status_code=400,
        )
        result = Request.get(given_any_url)
        assert_failure(result, value_is_instance_of=BadRequestError)
        assert result.value.status_code == 400
        assert result.value.headers == expected_headers
        assert result.value.content == expected_content



from typing import Callable

from petisco import RabbitMqConnector, ConfigEventSubscriber, RabbitMQEventSubscriber
from tests.modules.event.mothers.defaults import DEFAULT_ORGANIZATION, DEFAULT_SERVICE


class RabbitMqEventSubscriberMother:
    @staticmethod
    def default(main_handler: Callable, connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMQEventSubscriber(
            connector=connector,
            subscribers={
                "auth": ConfigEventSubscriber(
                    organization=DEFAULT_ORGANIZATION,
                    service=DEFAULT_SERVICE,
                    topic=f"{DEFAULT_SERVICE}-event",
                    handler=main_handler,
                )
            },
            connection_name="subscriber",
        )



import pytest

from petisco import Secret, NotHasSpecificLengthValueObjectError


@pytest.mark.unit
def test_should_construct_a_valid_secret():
    _ = Secret("9464a12f2186cdfe38fd1cfd797758c47c544e99")


@pytest.mark.unit
def test_should_construct_a_invalid_length_secret():
    with pytest.raises(NotHasSpecificLengthValueObjectError):
        _ = Secret("00F0020007700634222")


# @pytest.mark.unit
# def test_should_raise_an_error_when_construct_a_secret_with_non_hex_values():
#     with pytest.raises(SecretIsNotHexError):
#         _ = Secret("nont-valid-secret")



from time import sleep

import pytest
from meiga import isSuccess, isFailure, Result, Error

from petisco import Event
from petisco.event.queue.domain.specific_queue_config import SpecificQueueConfig
from petisco.event.shared.domain.event_subscriber import EventSubscriber

from petisco.fixtures.testing_decorators import testing_with_rabbitmq
from tests.modules.event.mothers.event_user_created_mother import EventUserCreatedMother
from tests.modules.event.mothers.queue_config_mother import QueueConfigMother
from tests.modules.event.mothers.rabbitmq_event_bus_mother import RabbitMqEventBusMother
from tests.modules.event.mothers.rabbitmq_event_configurer_mother import (
    RabbitMqEventConfigurerMother,
)
from tests.modules.event.mothers.rabbitmq_event_consumer_mother import (
    RabbitMqEventConsumerMother,
)
from tests.modules.event.spies.spy_events import SpyEvents


@pytest.mark.integration
@testing_with_rabbitmq
@pytest.mark.parametrize(
    "max_retries_allowed,expected_number_event_consumed,simulated_results",
    [
        (0, 1, [isFailure]),
        (1, 2, [isFailure, isSuccess]),
        (2, 3, [isFailure, isFailure, isSuccess]),
        (3, 4, [isFailure, isFailure, isFailure, isSuccess]),
        (4, 5, [isFailure, isFailure, isFailure, isFailure, isSuccess]),
    ],
)
def test_should_publish_consume_and_retry_event_from_rabbitmq_when_a_queue_is_configured_with_specific_parameters(
    max_retries_allowed, expected_number_event_consumed, simulated_results
):
    spy = SpyEvents()
    spy_specific = SpyEvents()

    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)
        result = simulated_results.pop(0)
        return result

    def assert_specific_consumer(event: Event) -> Result[bool, Error]:
        spy_specific.append(event)
        return isSuccess

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer, assert_specific_consumer],
        )
    ]

    specific_queue_config = SpecificQueueConfig(
        wildcard="*assert_specific_consumer",
        specific_retry_ttl=50,
        specific_main_ttl=75,
    )
    queue_config = QueueConfigMother.with_specific_queue_config(
        specific_queue_config, default_retry_ttl=100, default_main_ttl=100
    )

    configurer = RabbitMqEventConfigurerMother.with_queue_config(queue_config)
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    consumer = RabbitMqEventConsumerMother.with_max_retries(max_retries_allowed)
    consumer.add_subscribers(subscribers)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(1)
    spy.assert_first_event(event)
    spy.assert_last_event(event)
    spy.assert_count_by_event_id(event.event_id, expected_number_event_consumed)
    spy_specific.assert_number_unique_events(1)



import pytest
from meiga import Success

from petisco import controller_handler, TokenManager, InfoId, Petisco
from petisco.security.token_manager.accepted_token import AcceptedToken


@pytest.mark.unit
def test_should_execute_a_controller_without_any_input(given_any_petisco):
    @controller_handler()
    def my_controller():
        return Success("Hello Petisco")

    http_response = my_controller()

    assert http_response == ({"message": "OK"}, 200)


@pytest.mark.unit
def test_should_execute_a_controller_with_a_string_parameter(given_any_petisco):
    @controller_handler()
    def my_controller(param: str):
        assert isinstance(param, str)
        return Success("Hello Petisco")

    http_response = my_controller("param")

    assert http_response == ({"message": "OK"}, 200)


@pytest.mark.unit
def test_should_execute_a_controller_with_default_headers_parameter(given_any_petisco,):
    @controller_handler()
    def my_controller(headers):
        assert isinstance(headers, dict)
        return Success("Hello Petisco")

    http_response = my_controller()

    assert http_response == ({"message": "OK"}, 200)


@pytest.mark.unit
def test_should_execute_a_controller_with_default_token_info_parameter(
    given_any_petisco, given_any_token_type, given_any_headers_provider
):
    @controller_handler(
        token_manager=TokenManager(
            accepted_tokens=[AcceptedToken(token_type=given_any_token_type)]
        ),
        headers_provider=given_any_headers_provider,
    )
    def my_controller():
        return Success("Hello Petisco")

    http_response = my_controller()

    assert http_response == ({"message": "OK"}, 200)


@pytest.mark.unit
def test_should_execute_a_controller_without_token_info_parameter(
    given_any_petisco, given_any_token_type, given_any_headers_provider
):
    @controller_handler(
        token_manager=TokenManager(
            accepted_tokens=[AcceptedToken(token_type=given_any_token_type)]
        ),
        headers_provider=given_any_headers_provider,
    )
    def my_controller(info_id):
        assert isinstance(info_id, InfoId)
        return Success("Hello Petisco")

    http_response = my_controller()

    assert http_response == ({"message": "OK"}, 200)


@pytest.mark.unit
def test_should_execute_a_controller_without_receive_any_param(
    given_any_petisco, given_any_token_type, given_any_headers_provider
):
    @controller_handler(
        token_manager=TokenManager(
            accepted_tokens=[AcceptedToken(token_type=given_any_token_type)]
        ),
        headers_provider=given_any_headers_provider,
    )
    def my_controller(info_id: InfoId):
        assert isinstance(info_id, InfoId)
        return Success("Hello Petisco")

    http_response = my_controller()

    assert http_response == ({"message": "OK"}, 200)


@pytest.mark.unit
def test_should_execute_a_controller_getting_the_petisco(given_any_petisco):
    @controller_handler()
    def my_controller(petisco):
        assert isinstance(petisco, Petisco)
        return Success("Hello Petisco")

    http_response = my_controller()

    assert http_response == ({"message": "OK"}, 200)



import os

MYSQL_DATABASE_DEFAULT = "mysql_test"


class MySqlConnection:
    def __init__(
        self,
        server_name: str,
        driver: str,
        user: str,
        password: str,
        host: str,
        port: str,
        database_name: str,
        url: str,
    ):
        self.server_name = server_name
        self.driver = driver
        self.user = user
        self.password = password
        self.host = host
        self.port = port
        self.database_name = database_name
        self.url = url

    @staticmethod
    def create(
        server_name: str = "mysql",
        driver: str = "pymysql",
        user: str = "root",
        password: str = "root",
        host: str = "mysql",
        port: str = "3306",
        database_name: str = MYSQL_DATABASE_DEFAULT,
    ):
        url = (
            f"{server_name}+{driver}://{user}:{password}@{host}:{port}/{database_name}"
        )
        return MySqlConnection(
            server_name, driver, user, password, host, port, database_name, url
        )

    @staticmethod
    def create_local(database_name: str = MYSQL_DATABASE_DEFAULT):
        return MySqlConnection.create(
            host="localhost", port="3307", database_name=database_name
        )

    @staticmethod
    def from_environ():
        return MySqlConnection.create(
            "mysql",
            "pymysql",
            os.getenv("MYSQL_USER", "root"),
            os.getenv("MYSQL_PASSWORD", "root"),
            os.getenv("MYSQL_HOST", "mysql"),
            os.getenv("MYSQL_PORT", "3306"),
            os.getenv("MYSQL_DATABASE", MYSQL_DATABASE_DEFAULT),
        )



import json
from datetime import datetime
from typing import Dict

from meiga import Result, Error, Success

from petisco.webhooks.webhook.domain.webhook import Webhook
from petisco.http.request import Request
from petisco.webhooks.webhook.domain.webhook_result_id import WebhookResultId
from petisco.webhooks.webhook.domain.webhook_result import WebhookResult
from petisco.webhooks.webhook.infrastructure.body_digest_signature import (
    BodyDigestSignature,
)

TIME_FORMAT = "%Y-%m-%d %H:%M:%S.%f"


class WebhookSender:
    def __init__(self, organization: str):
        self.organization = organization

    def execute(self, webhook: Webhook, payload: Dict) -> Result[WebhookResult, Error]:
        webhook_delivery_id = WebhookResultId.generate()
        sent_on = datetime.utcnow()
        headers = self._get_headers(webhook, webhook_delivery_id, sent_on)
        auth = self._get_auth(webhook)

        result = Request.post(
            url=webhook.post_url,
            string_info=json.dumps(payload),
            headers=headers,
            auth=auth,
        )
        return Success(
            WebhookResult.create(
                webhook, webhook_delivery_id, sent_on, headers, payload, result
            )
        )

    def ping(self, webhook: Webhook) -> Result[WebhookResult, Error]:
        webhook_delivery_id = WebhookResultId.generate()
        sent_on = datetime.utcnow()
        headers = self._get_headers(webhook, webhook_delivery_id, sent_on)
        auth = self._get_auth(webhook)
        payload = {"is_ping": True}

        result = Request.post(
            url=webhook.post_url,
            string_info=json.dumps(payload),
            headers=headers,
            auth=auth,
        )

        return Success(
            WebhookResult.create(
                webhook, webhook_delivery_id, sent_on, headers, payload, result
            )
        )

    def _get_auth(self, webhook: Webhook):
        return BodyDigestSignature(
            secret=webhook.secret.get_bytes(),
            algorithm=webhook.algorithm.get_algorithm(),
            organization=self.organization,
            header=f"X-{self.organization}-Signature",
        )

    def _get_headers(
        self, webhook: Webhook, webhook_delivery_id: WebhookResultId, sent_on: datetime
    ):
        headers = {
            f"X-{self.organization}-Event": webhook.event_name,
            f"X-{self.organization}-Event-Version": webhook.event_version,
            f"X-{self.organization}-Delivery": webhook_delivery_id.value,
            f"X-{self.organization}-WebhookId": webhook.webhook_id.value,
            f"X-{self.organization}-Request-Timestamp": sent_on.strftime(TIME_FORMAT),
            f"User-Agent": f"{self.organization}-Hookshoot/",
            "apikey": webhook.api_key,
        }
        return headers
